#!/usr/bin/env python3

import imp
import os
import getopt,sys
import six
from typing import Any

from System.IO import StreamWriter
from System.Text import Encoding

from Deadline.Scripting import RepositoryUtils, FrameUtils, ClientUtils, PathUtils, StringUtils
from DeadlineUI.Controls.Scripting.DeadlineScriptDialog import DeadlineScriptDialog
from ThinkboxUI.Controls.Scripting.ButtonControl import ButtonControl
# Job Options UI
imp.load_source( "JobOptionsUI", os.path.join( RepositoryUtils.GetRepositoryPath( "submission/Common/Main", True ), "JobOptionsUI.py" ) )
import JobOptionsUI


mistikaProductsDict={ #procuct: [label, binaryName]
    "Boutique":["Mistika Boutique","mistika"],
    "VR":["Mistika VR","vr"]
    }
    
MISTIKA_DICT_LABEL=0  
MISTIKA_DICT_BINARY=1
    
mistikaProductKey=os.path.splitext(os.path.basename(__file__).replace("Mistika","").replace("Submission",""))[0]

def mistikaProductLabel():
    return mistikaProductsDict.get(mistikaProductKey)[MISTIKA_DICT_LABEL]
    
def mistikaProductLongKey():
    global mistikaProductKey
    return "Mistika{}".format(mistikaProductKey)
  
def mistikaProductBinary():
    return mistikaProductsDict.get(mistikaProductKey)[MISTIKA_DICT_BINARY]    



########################################################################
## Globals
########################################################################
scriptDialog = None  # type: DeadlineScriptDialog
settings = None
jobOptionsDialog = None  # type: JobOptionsUI.JobOptionsDialog
frameList = None
framesPerTask = None
submittedFromMonitor = None
imageRender = False
outputFilePath = ""
rndFileName = None
rndName = None
url= None
jobID= None
product=None
quiet=False

def processParams(argv):
    global rndFileName
    global rndName
    global frameRange
    global imageRender
    global outputFilePath
    global url
    global jobID
    global product
    global quiet
    try:
        opts, args = getopt.getopt(argv,"i:o:n:r:S:u:p:j:q",['ifile=','ofile=','name=','range=','sequence=','url=','product=','jobID=','quiet'])
    except getopt.GetoptError:
        print('invalid Params: Supported params [i:n:r:d:f:o:u:p:]:')
        print('-i [--ifile]   rndfile')
        print('-n [--name]    render name')
        print('-r [--range]   range')
        print('-S [--sequence] 1 if it is a sequence, 0 if it is a movie)')
        print('-o [--ofile]   output file')
        print('-u [--url]     feedback URL')
        print('-p [--product] product')
        print('-q [--quiet]')
        return False 
    for opt, arg in opts:
      if opt in ("-i", "--ifile"):
         rndFileName=arg
      elif opt in ("-n", "--name"):
         rndName = arg
      elif opt in ("-r", "--range"):
         frameRange = arg
      elif opt in ("-S", "--sequence"):
         imageRender = int(arg)>0
      elif opt in ("-o", "--ofile"):
         outputFilePath = arg
      elif opt in ("-u", "--url"):
         url = arg
      elif opt in ("-p", "--product"):
         product = arg
      elif opt in ("-j", "--jobID"):
         jobID = arg
      elif opt in ("-q", "--quiet"):
         quiet = True

    return True   

    
########################################################################
## Main Function Called By Deadline
########################################################################
def __main__( *args ):
    # type: (*Any) -> None
    global scriptDialog
    global settings
    global jobOptionsDialog
    global frameList
    global framesPerTask
    global submittedFromMonitor
    global imageRender
    global outputFilePath
    global rndFileName
    global rndName    
    global url
    global jobID
    global quiet
    submittedFromMonitor = not bool( args )
    if not submittedFromMonitor:
        if not processParams(args):        
            return -1

    scriptDialog = DeadlineScriptDialog()
    scriptDialog.SetTitle( "Submit {} Job To Deadline".format(mistikaProductLabel() ))
    scriptDialog.SetIcon( scriptDialog.GetIcon( mistikaProductLongKey()))
    jobOptionsDialog = JobOptionsUI.JobOptionsDialog( parentAppName="{}Monitor".format(mistikaProductLongKey()))

    scriptDialog.AddScriptControl( "JobOptionsDialog", jobOptionsDialog, "" )

    scriptDialog.AddGrid()
    scriptDialog.AddControlToGrid( "Separator1", "SeparatorControl", "{} Options".format(mistikaProductLabel()), 0, 0, colSpan=4 )

    scriptDialog.AddControlToGrid( "SceneLabel", "LabelControl", "Mistika File", 1, 0, "The Mistika file to be rendered.", False )
    scriptDialog.AddSelectionControlToGrid( "SceneBox", "FileBrowserControl", "", "Mistika Render Files (*.rnd);;All Files (*)", 1, 1, colSpan=3 )

    imageBox = scriptDialog.AddSelectionControlToGrid( "ImageBox", "CheckBoxControl", False, "Image Render", 2, 1, "Check this box if the selected .rnd file corresponds to an image sequence render." )
    scriptDialog.AddSelectionControlToGrid( "SubmitSceneBox", "CheckBoxControl", False, "Submit Mistika File With The Job", 2, 2, "If this option is enabled, the Mistika render file will be submitted with the job, and then copied locally to the Worker machine during rendering.", colSpan=2 )

    scriptDialog.AddControlToGrid( "FramesLabel", "LabelControl", "Frame List", 3, 0, "The list of frames to render.", False )
    framesBox = scriptDialog.AddControlToGrid( "FramesBox", "TextControl", "", 3, 1 )
    scriptDialog.AddControlToGrid( "ChunkSizeLabel", "LabelControl", "Frames Per Task", 3, 2, "This is the number of frames that will be rendered at a time for each job task.", False )
    chunkSizeBox = scriptDialog.AddRangeControlToGrid( "ChunkSizeBox", "RangeControl", 1, 1, 1000000, 0, 1, 3, 3 )
    scriptDialog.AddControlToGrid( "SeparatorFeedback", "SeparatorControl", "Mistika Feedback Options", 4, 0, colSpan=4 )
    scriptDialog.AddControlToGrid( "restServerLabel", "LabelControl", "Mistika Rest Server", 5, 0, "The Mistika Rest Server URL (used to feed back progress).", False )
    scriptDialog.AddControlToGrid( "restServer","TextControl", "", 5, 1, colSpan=3 ) 
    scriptDialog.AddControlToGrid( "jobIDLabel", "LabelControl", "Mistika Job ID", 6, 0, "The Mistika JobID used to identify the task for feedback.", False )
    scriptDialog.AddControlToGrid( "jobID","TextControl", "", 6, 1, colSpan=1 ) 

    scriptDialog.EndGrid()

    scriptDialog.AddGrid()
    scriptDialog.AddHorizontalSpacerToGrid( "HSpacer1", 0, 0 )
    submitButton = scriptDialog.AddControlToGrid( "SubmitButton", "ButtonControl", "Submit", 0, 1, expand=False )
    submitButton.ValueModified.connect( SubmitButtonPressed )

    closeButton = scriptDialog.AddControlToGrid( "CloseButton", "ButtonControl", "Close", 0, 2, expand=False )
    closeButton.ValueModified.connect( scriptDialog.closeEvent )
    closeButton.ValueModified.connect( jobOptionsDialog.closeEvent )
    scriptDialog.EndGrid()

    settings = ( "FramesBox", "SubmitSceneBox", "ChunkSizeBox", "ImageBox", "SceneBox","restServer","jobID" )
    scriptDialog.LoadSettings( GetSettingsFilename(), settings )
    scriptDialog.EnabledStickySaving( settings, GetSettingsFilename() )

    frameList = scriptDialog.GetValue( "FramesBox" )
    framesPerTask = scriptDialog.GetValue( "ChunkSizeBox" )

    # if submitting through Mistika then populate some fields
    if not submittedFromMonitor:
        jobOptionsDialog.SetValue( "NameBox", rndName )
        scriptDialog.SetValue( "SceneBox", rndFileName )
        scriptDialog.SetEnabled( "ImageBox", False )

        if imageRender:
            scriptDialog.SetValue( "ImageBox", True )
            # do this so the integrated submitter will have the frame range specified in Mistika when it appears
            frameList = frameRange
        else:
            scriptDialog.SetValue( "ImageBox", False )

        if url:
            scriptDialog.SetValue("restServer",url)
            
        if jobID !="":
            scriptDialog.SetValue("jobID",jobID)        
        
    ImageBoxChanged()

    # set the callbacks here so they don't get triggered when filling in the values initially
    imageBox.ValueModified.connect( ImageBoxChanged )
    framesBox.ValueModified.connect( FramesBoxChanged )
    chunkSizeBox.ValueModified.connect( ChunkSizeBoxChanged )
    if quiet:
        SubmitButtonPressed(None)
    else:
        scriptDialog.ShowDialog( True )

def GetSettingsFilename():
    # type: () -> str
    return os.path.join( ClientUtils.GetUsersSettingsDirectory(), "{}Settings.ini".format(mistikaProductLongKey()))

def ImageBoxChanged():
    # type: () -> None
    global scriptDialog
    global frameList
    global framesPerTask

    imageRenderChecked = scriptDialog.GetValue( "ImageBox" )

    scriptDialog.SetEnabled( "FramesLabel", imageRenderChecked )
    scriptDialog.SetEnabled( "FramesBox", imageRenderChecked )
    scriptDialog.SetEnabled( "ChunkSizeLabel", imageRenderChecked )
    scriptDialog.SetEnabled( "ChunkSizeBox", imageRenderChecked )

    if imageRenderChecked:
        scriptDialog.SetValue( "FramesBox", frameList )
        scriptDialog.SetValue( "ChunkSizeBox", framesPerTask )
    else:
        scriptDialog.SetValue( "FramesBox", "1" )
        scriptDialog.SetValue( "ChunkSizeBox", "1" )

def FramesBoxChanged():
    # type: () -> None
    global frameList

    if scriptDialog.GetEnabled( "FramesBox" ):
        frameList = scriptDialog.GetValue( "FramesBox" )

def ChunkSizeBoxChanged():
    # type: () -> None
    global framesPerTask

    if scriptDialog.GetEnabled( "ChunkSizeBox" ):
        framesPerTask = scriptDialog.GetValue( "ChunkSizeBox" )

def SubmitButtonPressed( *args ):
    # type: (*ButtonControl) -> None
    global scriptDialog
    global jobOptionsDialog
    global submittedFromMonitor
    global imageRender
    global outputFilePath
    global rndFileName
    global url
    global jobID

    warnings = []
    errors = []

    # Check if Mistika render files exist.
    sceneFiles = StringUtils.FromSemicolonSeparatedString( scriptDialog.GetValue( "SceneBox" ), False )
    if len( sceneFiles ) == 0 :
        errors.append( "No {} render file specified.".format(mistikaProductLabel()))

    submitScene = scriptDialog.GetValue( "SubmitSceneBox" )

    # Check if Mistika render file exists.
    for sceneFile in sceneFiles:
        if not os.path.isfile( sceneFile ):
            errors.append( "{} render file: {} does not exist.".format(mistikaProductLabel(),sceneFile))
        elif not submitScene and PathUtils.IsPathLocal( sceneFile ):
            warnings.append( "The {} render file: ".format(mistikaProductLabel()) + sceneFile + " is local and is not being submitted with the job, are you sure you want to continue?" )

    # Check if a valid frame range has been specified.
    frames = scriptDialog.GetValue( "FramesBox" )
    if not FrameUtils.FrameRangeValid( frames ):
        errors.append( "Frame range %s is not valid." % frames )

    if len( errors ) > 0:
        scriptDialog.ShowMessageBox( "The following errors were encountered:\n\n%s\n\nPlease resolve these issues and submit again.\n" % ( "\n\n".join( errors ) ), "Errors" )
        return

    if len( warnings ) > 0:
        result = scriptDialog.ShowMessageBox( "Warnings:\n\n%s\n\nDo you still want to continue?" % ( "\n\n".join( warnings ) ), "Warnings", ( "Yes", "No" ) )
        if result == "No":
            return

    jobOptions = jobOptionsDialog.GetJobOptionsValues()

    successes = 0
    failures = 0

    # Submit each Mistika render file separately.
    for sceneFile in sceneFiles:
        originalJobName = jobOptions[ "Name" ]
        if len( sceneFiles ) > 1:
            newJobName = "%s [%s]" % ( originalJobName, os.path.basename( sceneFile ) )
            jobOptions[ "Name" ] = newJobName

        # Create job info file.
        jobInfoFilename = os.path.join( ClientUtils.GetDeadlineTempPath(), "{}_job_info.job".format(mistikaProductLongKey()))
        writer = StreamWriter( jobInfoFilename, False, Encoding.Unicode )
        writer.WriteLine( "Plugin={}".format(mistikaProductLongKey()))

        for option, value in six.iteritems(jobOptions):
            writer.WriteLine( "%s=%s" % ( option, value ) )

        writer.WriteLine( "Frames=%s" % frames )
        #writer.WriteLine( "URL=%s" % url )
        #writer.WriteLine( "JobID=%s" % jobID )
        writer.WriteLine( "ChunkSize=%s" % scriptDialog.GetValue( "ChunkSizeBox" ) )

        if not submittedFromMonitor and sceneFile == rndFileName:
            writer.WriteLine( "OutputFilename0=%s" % outputFilePath.replace( "%06d", "######" ) )

        writer.Close()

        if len( sceneFiles ) > 1:
            # reset the job name for the next job to be submitted
            jobOptions[ "Name" ] = originalJobName

        # Create plugin info file.
        pluginInfoFilename = os.path.join( ClientUtils.GetDeadlineTempPath(), "{}_plugin_info.job".format(mistikaProductLongKey()))
        writer = StreamWriter( pluginInfoFilename, False, Encoding.Unicode )

        if not submitScene:
            writer.WriteLine( "SceneFile=%s" % sceneFile )

        writer.WriteLine( "ImageRender=%s" % scriptDialog.GetValue( "ImageBox" ) )
        writer.Close()

        # Setup the command line arguments.
        arguments = [ jobInfoFilename, pluginInfoFilename ]

        if submitScene:
            arguments.append( sceneFile )

        if len( sceneFiles ) == 1:
            results = ClientUtils.ExecuteCommandAndGetOutput( arguments )
            scriptDialog.ShowMessageBox( results, "Submission Results" )
        else:
            # Now submit the job.
            exitCode = ClientUtils.ExecuteCommand( arguments )
            if exitCode == 0:
                successes = successes + 1
            else:
                failures = failures + 1

    if len( sceneFiles ) > 1:
        scriptDialog.ShowMessageBox( "Jobs submitted successfully: %d\nJobs not submitted: %d" % (successes, failures), "Submission Results" )
